@@ -29,6 +29,7 @@ import os
 import sys
 import copy
 from ConfigParser import ConfigParser
+import json
 import subprocess
 import slapos.slap
 import netaddr
@@ -82,30 +83,33 @@ class Recipe:
       if 'default' in self.options:
         software_type = 'default'
       else:
-        raise zc.buildout.UserError("This software type isn't mapped. And"
+        raise zc.buildout.UserError("This software type isn't mapped. And "
                                     "there's no default software type.")
 
     instance_file_path = self.options[software_type]
 
     if not os.path.exists(instance_file_path):
-      raise zc.buildout.UserError("The specified buildout config file does not"
-                                  "exist.")
+      raise zc.buildout.UserError("The specified buildout config file %r does "
+                                  "not exist." % instance_file_path)
 
     buildout = ConfigParser()
     with open(instance_file_path) as instance_path:
       buildout.readfp(instance_path)
 
-    buildout.set('buildout', 'installed',
-                 '.installed-%s.cfg' % software_type)
+    buildout.set('buildout', 'installed', '.installed-%s.cfg' % self.name)
 
-    buildout.add_section('slap-parameter')
+    if not buildout.has_section('slap-parameter'):
+      buildout.add_section('slap-parameter')
     for parameter, value in self.parameter_dict.items():
-      buildout.set('slap-parameter', parameter, value)
+      if isinstance(value, str):
+        buildout.set('slap-parameter', parameter, value)
+      else:
+        buildout.set('slap-parameter', parameter, json.dumps(value))
 
     buildout.add_section('slap-network-information')
-    buildout.set('slap-network-information', 'local-ipv4', 
+    buildout.set('slap-network-information', 'local-ipv4',
                  self.getLocalIPv4Address())
-    buildout.set('slap-network-information', 'global-ipv6', 
+    buildout.set('slap-network-information', 'global-ipv6',
                  self.getGlobalIPv6Address())
 
     # Copy/paste slap_connection
@@ -117,7 +121,7 @@ class Recipe:
     work_directory = os.path.abspath(self.buildout['buildout'][
       'directory'])
     buildout_filename = os.path.join(work_directory,
-                                     'buildout-%s.cfg' % software_type)
+                                     'buildout-%s.cfg' % self.name)
     with open(buildout_filename, 'w') as buildout_file:
       buildout.write(buildout_file)
 
